using System;
using System.Collections.Generic;
using System.Text;
using Vevo.Domain;
using Vevo.Domain.Base;
using Vevo.Domain.Stores;
using Vevo.Shared.DataAccess;
using Vevo.Domain.Configurations;

namespace Vevo.Domain.Products
{
    // This is an Aggregate Root of Category. This aggregate has the following classes.
    //    - Category
    //    - LocaleCollection<CategoryLocale>
    //    - CategoryLocale
    public class Category : BaseCategory, IBaseCategory
    {
        #region Private

        // Members for properties
        private string _categoryID = "0";
        private string _parentCategoryID = "0";
        private string _categoryListLayoutPath = String.Empty;

        private bool GetCategoryInList(
            Culture culture,
            IList<Category> categoryList,
            string categoryID,
            out Category category )
        {
            foreach (Category categoryItem in categoryList)
            {
                if (categoryItem.CategoryID == categoryID)
                {
                    category = categoryItem;
                    return true;
                }
            }
            category = new Category( culture );
            return false;
        }

        private bool IsParentCategoryVisible( Category category, string rootID, Culture culture )
        {
            if (category.RootID == rootID)
            {
                if (category.IsEnabled)
                {
                    if (category.ParentCategoryID != rootID)
                    {
                        Category parentCategory = DataAccessContext.CategoryRepository.GetOne( culture, category.ParentCategoryID );
                        if (IsParentCategoryVisible( parentCategory, rootID, culture ))
                            return true;
                    }
                    else
                    {
                        return true;
                    }
                }
            }

            return false;
        }

        private string CreateFullCategoryPathWithEndText( string endText )
        {
            string result = endText;
            string root = RootID;
            string parent = ParentCategoryID;
            while ((parent != root) && (parent != "0"))
            {
                Category category = DataAccessContext.CategoryRepository.GetOne( DefaultCulture, parent );
                if (category != null)
                {
                    result = category.Name + " >> " + result;
                    parent = category.ParentCategoryID;
                }
            }
            return result;
        }

        #endregion


        #region Public Constructors

        public Category( Culture defaultCulture )
            : base( defaultCulture )
        {

        }

        public Category()
            : base()
        {

        }

        #endregion


        #region Public Properties

        public static Category Null
        {
            get { return NullCategory.Instance; }
        }


        public string CategoryID { get { return _categoryID; } set { _categoryID = value; } }
        public string ParentCategoryID { get { return _parentCategoryID; } set { _parentCategoryID = value; } }
        public string CategoryListLayoutPath { get { return _categoryListLayoutPath; } set { _categoryListLayoutPath = value; } }

        #endregion


        #region IClonable Members

        public override object Clone()
        {
            Category newObject = (Category) MemberwiseClone();
            newObject._locales = (LocaleCollection<BaseCategoryLocale>) Locales.Clone();

            return newObject;
        }

        #endregion

        #region ILocaleLoader Members

        public override IList<ILocale> GetLocales()
        {
            List<ILocale> result = new List<ILocale>();
            foreach (CategoryLocale locale in DataAccessContext.CategoryRepository.GetLocales( CategoryID ))
                result.Add( locale );

            return result;
        }

        #endregion


        #region Public Methods

        public override bool IsParentsEnable()
        {
            if (IsParentCategoryVisible( this, RootID, DefaultCulture ))
                return true;
            else
                return false;
        }

        public bool IsLeafCategory( IList<Category> allCategories )
        {
            foreach (Category category in allCategories)
            {
                if (category.ParentCategoryID == CategoryID)
                    return false;
            }
            return true;
        }

        public bool IsLeafCategory()
        {
            return DataAccessContext.CategoryRepository.IsLeaf( CategoryID );
        }

        public string CreateFullCategoryPath()
        {
            return CreateFullCategoryPathWithEndText( Name );
        }

        public string CreateFullCategoryPathParentOnly()
        {
            return CreateFullCategoryPathWithEndText( "" );
        }

        public string CreateParentCategoryPathList()
        {
            string result = String.Empty;
            string root = RootID;
            string parent = ParentCategoryID;

            while ((parent != root) && (parent != "0"))
            {
                Category category = DataAccessContext.CategoryRepository.GetOne( DefaultCulture, parent );
                if (category != null)
                {
                    result = result + category.Name + " - ";
                    parent = category.ParentCategoryID;
                }
            }
            return result;

        }

        public override IList<IBaseCategory> GetBreadcrumb()
        {
            IList<IBaseCategory> breadcrumb = new List<IBaseCategory>();

            breadcrumb.Insert( 0, this );

            string id = ParentCategoryID;
            while (id != "0")
            {
                IBaseCategory baseCategory = DataAccessContext.CategoryRepository.GetOne( DefaultCulture, id );

                breadcrumb.Insert( 0, baseCategory );
                Category category = (Category) baseCategory;
                id = category.ParentCategoryID;
            }

            return breadcrumb;
        }

        public string GetMetaDescription( Culture culture )
        {
            string metaDescription = SeoVariable.ReplaceCategoryVariable(
                       DataAccessContext.CategoryRepository.GetOne( culture, CategoryID ),
                       culture,
                       DataAccessContext.Configurations.GetValue(
                           culture.CultureID,
                           "DefaultCategoryMetaDescription",
                           DataAccessContext.StoreRepository.GetOne( DataAccessContext.StoreRetriever.GetCurrentStoreID() ) ) );

            return GetMetaDescription( culture, metaDescription );
        }

        public string GetMetaKeyword( Culture culture )
        {
            string metaKeyword = SeoVariable.ReplaceCategoryVariable(
                 DataAccessContext.CategoryRepository.GetOne( culture, CategoryID ),
                 culture,
                 DataAccessContext.Configurations.GetValue(
                     culture.CultureID,
                     "DefaultCategoryMetaKeyword",
                     DataAccessContext.StoreRepository.GetOne( DataAccessContext.StoreRetriever.GetCurrentStoreID() ) ) );

            return GetMetaKeyword( culture, metaKeyword );
        }

        #endregion

    }
}
