using System;
using System.Collections.Generic;
using log4net;
using Vevo.QuickBooks.States;

namespace Vevo.QuickBooks
{
    /// <summary>
    /// Summary description for QBWCSession
    /// </summary>
    public class QBWCSession
    {
        private const int MaximumSendRequestLoop = 100;

        private string _ticket;
        private QBWCState _currentState;

        private string _lastErrorMessage;
        private Dictionary<string, string> _qbFullName = new Dictionary<string, string>();
        private Dictionary<string, string> _qbEditSequenceData = new Dictionary<string, string>();
        private string _qbLogID;
        private DateTime _lastAccessTime;

        private ILog _logger;

        protected ILog Logger
        {
            get
            {
                if (_logger == null)
                {
                    _logger = LogManager.GetLogger( typeof( QBWCServices ) );
                }

                return _logger;
            }
        }


        public QBWCSession()
        {
            _ticket = Guid.NewGuid().ToString().ToLower();
            _lastErrorMessage = String.Empty;
            _lastAccessTime = DateTime.Now;
        }


        public string Ticket
        {
            get { return _ticket; }
        }

        public QBWCState CurrentState
        {
            get
            {
                if (_currentState == null)
                    _currentState = new CustomerVerifyRequestState( this );

                return _currentState;
            }
            set
            {
                Logger.Debug( "Change State to: " + value.GetType().ToString() );
                _currentState = value;
            }
        }

        public string LastErrorMessage
        {
            get
            {
                return _lastErrorMessage;
            }
            set
            {
                _lastErrorMessage = value;
            }
        }

        public string QBLogID
        {
            get
            { return _qbLogID; }
            set 
            { _qbLogID = value; }
        }

        public string OnSendRequestXml()
        {
            string xml;
            bool xmlReady = false;
            int count = 0;
            do
            {
                // If a state returns an empty string, that means it wants to skip to another state.
                // Then, we try to call OnSendRequstXml again.
                xml = CurrentState.OnSendRequestXml( out xmlReady );
                count++;
            }
            while (!xmlReady && count < MaximumSendRequestLoop);

            return xml;
        }

        public int OnReceiveResponseXml( string responseXml )
        {
            return CurrentState.OnReceiveResponseXml( responseXml );
        }

        public void AddQBFullNameList( string key, string value )
        {
            if (!IsExistName( key ))
                _qbFullName.Add( key, value );
        }

        public Dictionary<string, string> GetNameList()
        {
            return _qbFullName;
        }

        public void AddQBEditSequenceList( string key, string value )
        {
            if (!_qbEditSequenceData.ContainsKey( key ))
                _qbEditSequenceData.Add( key, value );
        }

        public Dictionary<string, string> GetEditSequenceList()
        {
            return _qbEditSequenceData;
        }

        public void ClearList()
        {
            _qbFullName.Clear();
            _qbEditSequenceData.Clear();
        }

        public bool IsExistName( string name )
        {
            return _qbFullName.ContainsKey( name );
        }

        public bool IsExpired( int idleTimeoutSeconds )
        {
            TimeSpan lastAccess = DateTime.Now.Subtract( _lastAccessTime );
            return (lastAccess.CompareTo( new TimeSpan( 0, 0, idleTimeoutSeconds ) ) >= 0);
        }
    }
}
