using System;
using System.Collections;
using System.Collections.Specialized;
using System.Net.Mail;
using System.Text;
using System.Web;
using System.Web.Security;
using System.Web.SessionState;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using Vevo.DataAccessLib;
using Vevo.Domain;
using Vevo.Shared.DataAccess.Tester;
using Vevo.Domain.Stores;
using Vevo.Domain.Products;
using System.Collections.Generic;
using Vevo.Shared.SystemServices;
using Vevo.Shared.Domain.Logging;
using System.IO;

namespace Vevo.WebAppLib
{
    public class EmailDetails
    {
        public string From;
        public string FromName;
        public string To;
        public string Subject;
        public string Body;
    }

    /// <summary>
    /// Summary description for Utilities
    /// </summary>
    public static class WebUtilities
    {
        #region Private
        private static EmailDetails LatestEmailSession
        {
            get
            {
                if (HttpContext.Current.Session["LatestEmail"] == null)
                    return new EmailDetails();
                else
                    return (EmailDetails) HttpContext.Current.Session["LatestEmail"];
            }
            set
            {
                HttpContext.Current.Session["LatestEmail"] = value;
            }
        }

        private static void SaveEmailToSession(
            MailAddress from, string to, string subject, string body )
        {
            EmailDetails email = new EmailDetails();
            email.From = from.Address;
            email.FromName = from.DisplayName;
            email.To = to;
            email.Subject = subject;
            email.Body = body;

            LatestEmailSession = email;
        }

        private static MailMessage CreateMailMessage(
            MailAddress from, string to, string subject, string body )
        {
            MailMessage mailMessage = new MailMessage();

            string[] toList = to.Split( new char[] { ';', ',' } );
            foreach (string recipient in toList)
            {
                if (!String.IsNullOrEmpty( recipient.Trim() ))
                    mailMessage.To.Add( new MailAddress( recipient ) );
            }
            mailMessage.From = from;
            mailMessage.Subject = subject;
            mailMessage.Body = body;

            return mailMessage;
        }

        private static MailMessage CreateHtmlMailMessage(
            MailAddress from, string to, string subject, string body )
        {
            MailMessage mailMessage = CreateMailMessage( from, to, subject, body );
            mailMessage.IsBodyHtml = true;
            mailMessage.BodyEncoding = System.Text.Encoding.UTF8;
            mailMessage.SubjectEncoding = System.Text.Encoding.UTF8;

            return mailMessage;
        }

        private static void TieLoginWithButton( Page page, Login login, Control button )
        {
            Control userName = login.FindControl( "UserName" );
            Control password = login.FindControl( "Password" );
            Control checkBox = login.FindControl( "RememberMe" );

            WebUtilities.TieButton( page, userName, button );
            WebUtilities.TieButton( page, password, button );
            WebUtilities.TieButton( page, checkBox, button );
        }

        private static SmtpClient CreateSmtpClient()
        {
            return new SmtpClient(
                DataAccessContext.Configurations.GetValue( "SmtpServer" ),
                DataAccessContext.Configurations.GetIntValue( "SmtpPort" ) );
        }

        private static SmtpClient CreateSmtpClient( Store store )
        {
            return new SmtpClient(
                DataAccessContext.Configurations.GetValue( "SmtpServer", store ),
                DataAccessContext.Configurations.GetIntValue( "SmtpPort", store ) );
        }

        private static void SetAuthentication( SmtpClient mailClient )
        {
            System.Net.NetworkCredential smtpUserInfo =
                new System.Net.NetworkCredential(
                DataAccessContext.Configurations.GetValue( "SmtpUserName" ),
                DataAccessContext.Configurations.GetValue( "SmtpPassword" )
                );
            mailClient.UseDefaultCredentials = false;
            mailClient.Credentials = smtpUserInfo;
        }

        private static void SetAuthentication( SmtpClient mailClient, Store store )
        {
            System.Net.NetworkCredential smtpUserInfo =
                new System.Net.NetworkCredential(
                DataAccessContext.Configurations.GetValue( "SmtpUserName", store ),
                DataAccessContext.Configurations.GetValue( "SmtpPassword", store )
                );

            mailClient.UseDefaultCredentials = false;
            mailClient.Credentials = smtpUserInfo;
        }

        private static void SplitPathAndQuery( string url, out string path, out string query )
        {
            int index = url.IndexOf( '?' );
            if (index >= 0)
            {
                path = url.Substring( 0, index );
                query = url.Substring( index + 1 );
            }
            else
            {
                path = String.Empty;
                query = url;
            }
        }

        private static string GetProductID( string pathAndQuery )
        {

            if (!String.IsNullOrEmpty( pathAndQuery ) && pathAndQuery.ToLower().Contains( "productid=" ))
                return pathAndQuery.Substring( "/Product.aspx?productid=".Length );
            else
                return DataAccessContext.ProductRepository.GetProductIDFromUrlName( GetProductName( pathAndQuery ) );

        }

        private static string GetProductName( string pathAndQuery )
        {
            if (String.IsNullOrEmpty( pathAndQuery ))
                return String.Empty;
            else
                return pathAndQuery.Substring( "/Product.aspx?ProductName=".Length );
        }

        private static bool IsCanonStoreIDAvaliable( IList<string> storeIDList, string canonStoreID )
        {
            foreach (string availableStoreID in storeIDList)
            {
                if (canonStoreID.Equals( availableStoreID ))
                {
                    return true;
                }
            }
            return false;
        }

        private static IList<string> GetAvailableStoreIDList( StoreRetriever storeRetriever, Culture culture, string pageAndQueryUrl )
        {
            string storeID = storeRetriever.GetCurrentStoreID();
            if (pageAndQueryUrl.ToLower().Contains( "-details.aspx" ) ||
                pageAndQueryUrl.ToLower().Contains( "product.aspx" ))
            {
                string productID = GetProductID( pageAndQueryUrl );
                Product product = DataAccessContext.ProductRepository.GetOne( culture, productID, storeID );
                return product.GetAvailableStoreIDList();
            }

            if (pageAndQueryUrl.ToLower().Contains( "-list.aspx" ) ||
                pageAndQueryUrl.ToLower().Contains( "catalog.aspx" ))
            {
                string rootCategoryID = DataAccessContext.Configurations.GetValue( "RootCategory", storeRetriever.GetStore() );
                IList<Category> categoryList = DataAccessContext.CategoryRepository.GetByRootID(
                    culture, rootCategoryID, "CategoryID", BoolFilter.ShowAll );
                if (categoryList != null)
                {
                    return categoryList[0].GetAvailableStoreIDList();
                }
            }

            if (pageAndQueryUrl.ToLower().Contains( "-items.aspx" ) ||
                pageAndQueryUrl.ToLower().Contains( "department.aspx" ))
            {
                string rootDepartmentID = DataAccessContext.Configurations.GetValue( "RootDepartment", storeRetriever.GetStore() );
                IList<Department> departmentList = DataAccessContext.DepartmentRepository.GetByRootID(
                    culture, rootDepartmentID, "DepartmentID", BoolFilter.ShowAll );
                if (departmentList != null)
                {
                    return departmentList[0].GetAvailableStoreIDList();
                }
            }

            return new List<string>();

        }
        #endregion


        public static void SendMail( string from, string to, string subject, string body, Store store )
        {
            SendMail(
                new MailAddress( from ),
                to,
                subject,
                body, store );
        }

        public static void SendMail( string from, string to, string subject, string body )
        {
            SendMail(
                new MailAddress( from ),
                to,
                subject,
                body, new StoreRetriever().GetStore() );
        }

        public static void SendHtmlMail( string from, string to, string subject, string body, Store store )
        {
            SendHtmlMail(
                new MailAddress( from ),
                to,
                subject,
                body, store );
        }

        public static void SendHtmlMail( MailAddress from, string to, string subject, string body )
        {
            SendHtmlMail(
                from,
                to,
                subject,
                body, new StoreRetriever().GetStore() );
        }

        public static void SendHtmlMail( string from, string to, string subject, string body )
        {
            SendHtmlMail(
                new MailAddress( from ),
                to,
                subject,
                body, new StoreRetriever().GetStore() );
        }

        public static void SendMail( MailAddress from, string to, string subject, string body )
        {
            SendMail(
                from,
                to,
                subject,
                body,
                new StoreRetriever().GetStore() );
        }

        public static void SendMail( MailAddress from, string to, string subject, string body, Store store )
        {
            MailMessage mailMessage = CreateMailMessage( from, to, subject, body );

            SmtpClient mailClient = CreateSmtpClient( store );

            if (DataAccessContext.Configurations.GetBoolValue( "SmtpAuthenRequired", store ))
            {
                SetAuthentication( mailClient, store );
            }
            try
            {
                mailClient.EnableSsl = DataAccessContext.Configurations.GetBoolValue( "SmtpSslRequired", store );
            }
            catch (Exception e)
            {
                LogError( e );

                mailClient.EnableSsl = false;
            }
            mailClient.Send( mailMessage );
            SaveEmailToSession( from, to, subject, body );
        }

        public static void SendHtmlMail( MailAddress from, string to, string subject, string body, Store store )
        {
            MailMessage mailMessage = CreateHtmlMailMessage( from, to, subject, body );

            SmtpClient mailClient = CreateSmtpClient( store );

            if (DataAccessContext.Configurations.GetBoolValue( "SmtpAuthenRequired", store ))
            {
                SetAuthentication( mailClient, store );
            }
            try
            {
                mailClient.EnableSsl = DataAccessContext.Configurations.GetBoolValue( "SmtpSslRequired", store );
            }
            catch (Exception e)
            {
                LogError( e );
                mailClient.EnableSsl = false;
            }

            mailClient.Send( mailMessage );
            SaveEmailToSession( from, to, subject, body );
        }

        public static EmailDetails GetLatestEmail()
        {
            return LatestEmailSession;
        }

        public static void LogError( Exception ex )
        {
            ExceptionLogPurger purger = new ExceptionLogPurger( new FileManager() );
            purger.PurgeExpiredLogs();

            Vevo.Shared.Domain.Logging.ExceptionLog exceptionLog = new Vevo.Shared.Domain.Logging.ExceptionLog( ex, new FileManager() );
            exceptionLog.Save();
        }

        public static string GetVisitorIP()
        {
            string ip = HttpContext.Current.Request.ServerVariables["HTTP_X_FORWARDED_FOR"];
            if (String.IsNullOrEmpty( ip ))
                ip = HttpContext.Current.Request.ServerVariables["REMOTE_ADDR"];

            return ip;
        }

        public static bool IsCurrentLoginCustomers()
        {
            string[] rolesType = Roles.GetRolesForUser();
            if (rolesType.Length == 0)
                return false;
            if (rolesType[0] == "Customers")
                return true;
            else
                return false;
        }

        // Configures what button to be clicked when the uses presses Enter in a
        // text box. The text box doesn't have to be a TextBox control, but it must
        // be derived from either HtmlControl or WebControl, and the HTML control it
        // generates should accept an 'onkeydown' attribute. The HTML generated by
        // the button must support the 'Click' event
        public static void TieButton(
            Page page,
            Control TextBoxToTie,
            Control ButtonToTie )
        {
            // Init jscript
            string jsString = "";

            // Check button type and get required jscript
            if (ButtonToTie is LinkButton)
            {
                jsString = "if ((event.which && event.which == 13) || (event.keyCode && event.keyCode == 13)) {"
                           + page.ClientScript.GetPostBackEventReference
                           ( ButtonToTie, "" ).Replace( ":", "$" ) + ";return false;} else return true;";
            }
            else if (ButtonToTie is ImageButton)
            {
                jsString = "if ((event.which && event.which == 13) || (event.keyCode && event.keyCode == 13)) {"
                           + page.ClientScript.GetPostBackEventReference
                           ( ButtonToTie, "" ).Replace( ":", "$" ) + ";return false;} else return true;";
            }
            else
            {
                jsString = "if ((event.which && event.which == 13) || "
                           + "(event.keyCode && event.keyCode == 13)) {document."
                           + "forms[0].elements['"
                           + ButtonToTie.UniqueID.Replace( ":", "_" )
                           + "'].click();return false;} else return true; ";
            }

            // Attach jscript to the onkeydown attribute-we have to cater for HtmlControl or WebControl
            if (TextBoxToTie is HtmlControl)
            {
                ((HtmlControl) TextBoxToTie).Attributes.Add( "onkeydown", jsString );
            }
            else if (TextBoxToTie is WebControl)
            {
                ((WebControl) TextBoxToTie).Attributes.Add( "onkeydown", jsString );
            }
        }

        // Tie user name and password text boxes of Login control to login button.
        public static void TieLoginControl( Page page, Login login )
        {
            Control button = login.FindControl( "LoginButton" );
            TieLoginWithButton( page, login, button );
        }

        // Tie user name and password text boxes of Login control to login button.
        public static void TieLoginControlImageButton( Page page, Login login )
        {
            Control button = login.FindControl( "uxLoginImageButton" );
            TieLoginWithButton( page, login, button );
        }

        public static string UrlAddQueryVariable(
            string originalUrl,
            string queryName,
            string queryValue )
        {
            string removed = UrlRemoveQueryVariable( originalUrl, queryName );

            StringBuilder sb = new StringBuilder();
            sb.Append( removed );
            if (removed.Contains( "?" ))
                sb.Append( '&' );
            else
                sb.Append( '?' );
            sb.AppendFormat( "{0}={1}", queryName, queryValue );

            return sb.ToString();
        }

        public static string UrlRemoveQueryVariable(
            string originalUrl,
            string queryName )
        {
            string path;
            string query;

            SplitPathAndQuery( originalUrl, out path, out query );

            StringBuilder sb = new StringBuilder();
            sb.Append( path );

            string[] urlArray = query.Split(
                new char[] { '&' }, StringSplitOptions.RemoveEmptyEntries );

            bool appendQuestion = false;
            for (int i = 0; i < urlArray.Length; i++)
            {
                if (!urlArray[i].StartsWith( queryName + "=", StringComparison.OrdinalIgnoreCase ))
                {
                    if (!appendQuestion)
                    {
                        sb.Append( '?' );
                        appendQuestion = true;
                    }
                    else
                    {
                        sb.Append( '&' );
                    }
                    sb.Append( urlArray[i] );
                }
            }

            return sb.ToString();
        }

        public static string GetApplicationRelativePath()
        {
            HttpRequest request = HttpContext.Current.Request;

            int appPathLength = request.ApplicationPath.Length;
            return request.Url.AbsolutePath.Substring( appPathLength );
        }

        public static string ReplaceNewLine( string text )
        {
            return text.Replace( "\n", "<br/>" );
        }

        public static void ClearCacheWithPrefix( string prefix )
        {
            StringCollection keysToRemove = new StringCollection();

            IDictionaryEnumerator cacheEnum = HttpRuntime.Cache.GetEnumerator();
            while (cacheEnum.MoveNext())
            {
                string key = cacheEnum.Key.ToString();
                if (key.StartsWith( prefix, StringComparison.OrdinalIgnoreCase ))
                    keysToRemove.Add( key );
            }

            foreach (string item in keysToRemove)
            {
                HttpRuntime.Cache.Remove( item );
            }
        }

        public static Control FindControlRecursive( Control root, string id )
        {
            if (root.ID == id)
            {
                return root;
            }

            foreach (Control c in root.Controls)
            {
                Control t = FindControlRecursive( c, id );
                if (t != null)
                {
                    return t;
                }
            }

            return null;
        }

        public static void ButtonAddConfirmation(
            WebControl control,
            string message )
        {
            control.Attributes.Add(
                "onclick",
                "if (confirm('" +
                message +
                "')) {} else {return false}" );
        }

        public static void PageAddMeta(
            Page page,
            string name,
            string content )
        {
            HtmlMeta meta = new HtmlMeta();
            meta.Name = name;
            if (!String.IsNullOrEmpty( content ))
                meta.Content = content;
            else
                meta.Content = " ";

            page.Header.Controls.Add( meta );
        }

        public static string GetLocalPath( string relativePath )
        {
            if (relativePath.Trim().StartsWith( "~/" ))
                return HttpContext.Current.Server.MapPath( relativePath.Trim() );
            else
                return HttpContext.Current.Server.MapPath( "~/" + relativePath.Trim() );
        }

        public static void CreateExportFile( HttpApplication app, MemoryStream stream, string filename, out Exception ex )
        {
            CreateExportFile( app, (Stream)stream, filename, out ex );
        }

        public static void CreateExportFile( HttpApplication app, Stream stream, string filename, out Exception ex )
        {
            //Buffer to read 10K bytes in chunk:
            byte[] buffer = new byte[10000];

            //Clean up the memory stream
            stream.Flush();

            //Length of the file:
            int length;

            //Total bytes to read:
            long dataToRead;

            try
            {
                stream.Position = 0;
                dataToRead = stream.Length;

                app.Response.Clear();

                app.Response.ContentType = "application/octet-stream";
                app.Response.AddHeader( "Content-Disposition", "attachment; filename=\"" + filename + "\"" );
                app.Response.AddHeader( "Content-Length", stream.Length.ToString() );
                //Response.BinaryWrite( byteArray );

                // Read the bytes.
                while (dataToRead > 0)
                {
                    // Verify that the client is connected.
                    if (app.Response.IsClientConnected)
                    {
                        // Read the data in buffer.
                        length = stream.Read( buffer, 0, 10000 );

                        // Write the data to the current output stream.
                        app.Response.OutputStream.Write( buffer, 0, length );

                        // Fluse the data to the HTML output.
                        app.Response.Flush();

                        buffer = new Byte[10000];
                        dataToRead = dataToRead - length;
                    }
                    else
                    {
                        //prevent infinite loop if user disconnects
                        dataToRead = -1;
                    }
                }
                app.CompleteRequest();
            }

            catch (Exception exception)
            {
                ex = exception;
            }
            finally
            {
                if (stream != null)
                {
                    stream.Close();
                }
            }
            ex = null;
        }

        public static void PageCannonicalization( Page page, string pageAndQueryUrl, Culture culture )
        {
            bool canonicalizationEnabled = DataAccessContext.Configurations.GetBoolValue( "CanonicalizationLayout" );
            string canonicalizationStoreID = DataAccessContext.Configurations.GetValue( "CanonicalizationStore" );

            if (canonicalizationEnabled)
            {
                StoreRetriever storeRetriever = new StoreRetriever();

                IList<string> storeIDList = GetAvailableStoreIDList( storeRetriever, culture, pageAndQueryUrl );
                string storeID = storeRetriever.GetCurrentStoreID();

                string canonStoreID = canonicalizationStoreID;
                if (!IsCanonStoreIDAvaliable( storeIDList, canonicalizationStoreID ))
                {
                    canonStoreID = storeIDList[0];
                }

                if (!canonStoreID.Equals( storeID ))
                {
                    HtmlLink linkTag = new HtmlLink();
                    linkTag.Attributes.Add( "rel", "canonical" );

                    try
                    {
                        linkTag.Href = storeRetriever.GetStorefrontUrlByStoreID( canonStoreID ) + pageAndQueryUrl;
                    }
                    catch
                    {
                        linkTag.Href = storeRetriever.GetStorefrontUrlByStoreID( "1" ) + pageAndQueryUrl;
                    }
                    page.Header.Controls.Add( linkTag );
                    page.Header.Controls.Add( new LiteralControl( Environment.NewLine ) );
                }
            }
        }
    }
}